/*
 * java2html.cc
 *
 * java2html
 * generates html code from Java code
 * Copyright (C) 1999  Lorenzo Bettini, lorenzo.bettini@penteres.it
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */                         

#include "version.h"
#include "java2html.h"

#include "colors.h"
#include "tags.h"
#include "keys.h"

#include "textgen.h"
#include "decorators.h"
#include "generators.h"

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>

/* global symbols */

char *inputFileName ; /* what we're reading  */
ostream* sout ;

int tabSpaces = 0 ; // space to substitue to tabs

extern int yylex() ;
extern int parseTags() ;

int main( int argc, char * argv[] ) {
  FILE *in = NULL;
  int entire_doc = 0 ; // we want a real html doc
  int otherArgs ;
  char *argString ;

  /* initialization of global symbols */
  inputFileName = NULL ;

  cerr << JAVA2HTML_VERSION << endl ;

  if (argc == 2 && 
      (strcmp( argv[1], "-help" ) == 0 || 
       strcmp( argv[1], "?" ) == 0 ) ) {
      cerr << "Syntax : java2html {infile} {outfile} {-doc} {-tab=#}" << endl ;
      cerr << "  -doc   : " << 
          "create a complete html doc (with <HTML>, <BODY>...\n"; 
      cerr << "  -tab=# : " <<
          "substitute a tab char with # spaces" << endl ;
      exit(0);
  }

  parseTags() ;
  createGenerators() ;

  if ( argc > 1 )
    {
      if ( ! in ) {
	in = freopen (argv[1], "r", stdin);
	if (in == NULL) {
	  cerr << "\n\nError opening " << argv[1] << endl ;
	  exit(1);
	}
      }
      inputFileName = strdup( argv[1] ) ;
      if ( argc > 2 ) {
        sout = new ofstream( argv[2] ) ;
        if ( ! sout ) {
          cerr << "Error in creating " << argv[2] << endl ;
          exit(1) ;
        }
      }
      else
	sout = &cout ;
    }
  else
    sout = &cout ; /* in this version only prints to standard output */
  
  otherArgs = 3 ;
  char *tabString ;
  while ( otherArgs < argc ) {
    argString = argv[ otherArgs ] ;
    if ( ( strcmp( argString, "-doc" ) == 0 ) )
      entire_doc = 1 ;
    else if ( strncmp( argString, "-tab", 4 ) == 0 &&
              (tabString = strchr( argString, '=' )) != 0 )
      tabSpaces = atoi( &tabString[1] ) ;
    else
      cerr << "Warning: unknown option " << argString << endl ;
    ++otherArgs ;
  }

  if ( entire_doc )
      print_top() ;  

  cerr << "translating Java code... " ;
  generateln( "<pre>" ) ;
  generateln( "<tt>" ) ;
  yylex() ;
  generateln( "</tt>" ) ;
  generateln( "</pre>" ) ;
  cerr << "done !" << endl ;

  if ( entire_doc )
      print_bottom() ;

  return 0 ;
}



void print_top() {
    generateln( "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">" ) ;
    generateln( "<html>" ) ;
    generateln( "<head>" ) ;
    generateln( "<meta http-equiv=\"Content-Type\"" ) ;
    generateln( "content=\"text/html; charset=iso-8859-1\">" ) ;
    generate( "<meta name=\"GENERATOR\" content=\"" ) ;
    generate( JAVA2HTML_VERSION ) ;
    generate( "\nby Lorenzo Bettini, lorenzo.bettini@penteres.it" ) ;
    generateln( "\">" ) ;
    generate( "<title>" ) ;
    generate( ( inputFileName ? inputFileName : "Java source file" ) ) ;
    generateln( "</title>" ) ;
    generateln( "</head>" ) ;
    generateln( "<body bgcolor=\"#FFFFFF\" text=\"#000000\" link=\"#0000EE\"" ) ;
    generateln( "vlink=\"#551A8B\" alink=\"#FF0000\">" ) ;
}

void print_bottom() {
    generateln( "</body>" ) ;
    generateln( "</html>" ) ;
}

void generate( char *s ) {
  GlobalGenerator->generate(s) ;
}

void generateln( char *s ) {
  GlobalGenerator->generateln(s) ;
}

void generateNewLine() {
  generateln( "" ) ;
}

void generateTab() {
  if ( tabSpaces )
    for ( register i = 0 ; i < tabSpaces ; ++i )
      generate( SPACE_CHAR ) ;
  else
      generate( "\t" ) ;
}

void startComment( char *s ) 
{
    CommentGenerator->beginText(s) ;
}

void endComment( char *s )
{
    CommentGenerator->endText(s) ;
}

void generateComment( char *s ) {
  CommentGenerator->generateEntire(s) ;
}

void startString( char *s )
{
    StringGenerator->beginText(s) ;
}

void endString( char *s )
{
    StringGenerator->endText(s) ;
}

void generateString( char *s ) {
  StringGenerator->generateEntire(s) ;
}

void generateKeyWord( char *s ) {
    KeywordGenerator->generateEntire(s) ;
}

void generateBaseType( char *s ) {
    TypeGenerator->generateEntire(s) ;
}

void generateNumber( char *s ) {
    NumberGenerator->generateEntire(s) ;
}

void startTAG( char *tag, char *attr, char *val ) {
  (*sout) << "<" << tag ;
  if ( attr && val )
    (*sout) << " " << attr << "=" << val ;
  (*sout) << ">" ;
}

void endTAG( char *tag ) {
  (*sout) << "</" << tag << ">" ;
}

void startColor( char *color ) {
  startTAG( FONT_TAG, COLOR_TAG, color ) ;
}

void endColor() {
  endTAG( FONT_TAG ) ;
}
